#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#define BUF_CMD           4096
#define JAVA_CMD          "java"
#define JAVA_CGI_CLASS    "foo.bar.CGI"

void error(const char* msg);
char *get(void);
void post(char **buf);
void run_java_cgi(const char *query);

int main(void) {
  char *request_method = (char *)getenv("REQUEST_METHOD");

  if(request_method == NULL) {
    error("undefined environment variable to 'REQUEST_METHOD'.");
  }

  char *query;

  if(strcmp(request_method, "GET") == 0) {
    query = get();
  } else if(strcmp(request_method, "POST") == 0) {
    post(&query);
  } else {
     query = "";
  }

  run_java_cgi(query);

  return EXIT_SUCCESS;
}

char *get(void) {
  char *query_string = (char *)getenv("QUERY_STRING");

  if(query_string == NULL) {
    error("undefined environment variable to 'QUERY_STRING'.");
  }

  return query_string;
}

void post(char **buf) {
  char *content_length = getenv("CONTENT_LENGTH");

  if(content_length == NULL) {
    error("undefined environment variable to 'CONTENT_LENGTH'.");
  }

  int len = atoi(content_length);
  *buf = (char *)malloc(len + 1);

  if(*buf == NULL) {
    error("cannot allocate memory.");
  }

  if(fread(*buf, len, 1, stdin) < 1) {
    error("cannot read stdin.");
  }

  (*buf)[len] = '\0';
}

void run_java_cgi(const char *query) {
  char cmd[BUF_CMD];
  cmd[0] = '\0';

  char *java_home = getenv("JAVA_HOME");

  if(java_home != NULL) {
    strcat(cmd, java_home);
    strcat(cmd, "/bin/");
  }

  strcat(cmd, JAVA_CMD);

  char *java_cgi_cp = getenv("JAVA_CGI_CP");

  if(java_cgi_cp != NULL) {
    strcat(cmd, " -cp ");
    strcat(cmd, java_cgi_cp);
  }

  strcat(cmd, " ");

  char *java_cgi_class = getenv("JAVA_CGI_CLASS");

  if(java_cgi_class != NULL) {
    strcat(cmd, java_cgi_class);
  } else {
    strcat(cmd, JAVA_CGI_CLASS);
  }

  char *path_info = (char *)getenv("PATH_INFO");

  if(path_info != NULL) {
    strcat(cmd, " ");
    strcat(cmd, path_info);
  }

  FILE *java_cgi;
  java_cgi = popen(cmd, "w");

  if(java_cgi == NULL) {
    error("cannot execute java command.");
  }

  fprintf(java_cgi, "%s", query); 

  int code = pclose(java_cgi);

  if(code != 0) {
    const char *template = "exit failure to '%s'.";
    char msg[BUF_CMD + strlen(template)];
    sprintf(msg, template, cmd);
    error(msg);
  }
}

void error(const char* msg) {
  printf("Content-Type: text/html\n\n");
  printf("<html>\n");
  printf("  <head>\n");
  printf("    <title>ERROR - [Java/CGI]</title>\n");
  printf("  </head>\n");
  printf("  <body>\n");
  printf("    %s\n", msg);
  printf("  </body>\n");
  printf("</html>\n");
  exit(EXIT_FAILURE);
}
