package foo.bar;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import foo.bar.impl.CGIConfImpl;

class CGI {

	private static final int BUF_LEN = 256;

	private static final String CONFIG_FILE = "jcgi.properties";

	private static final String CONFIG_PREFIX = "jcgi.";

	public static void main(String[] args) {

		try {
			StringWriter outBuf = new StringWriter();
			PrintWriter writer = new PrintWriter(outBuf);

			writer.println("Content-Type: text/html");
			writer.println();

			CGIConf config = new CGIConfImpl(CONFIG_FILE, CONFIG_PREFIX);
			String enc = config.getEncoding();

			String extPath = getExtendedPath(args);

			String query = getQuery();
			Map params = getParams(query, enc);

			CGIRunner runner = config.getRunner();

			runner.setConfig(config);
			runner.setExtendedPath(extPath);
			runner.setParams(params);
			runner.run(writer);

			writer.flush();

			System.out.println(outBuf.toString());

			writer.close();
		} catch (Exception e) {
			System.out.println("Content-Type: text/plain");
			System.out.println();
			e.printStackTrace(System.out);
			System.out.println(System.getProperties());
			System.out.println();
		}
	}

	private static String getQuery() throws IOException {
		InputStreamReader stdin = new InputStreamReader(System.in);
		BufferedReader bufin = new BufferedReader(stdin);

		StringBuffer strBuf = new StringBuffer();
		char[] cbuf = new char[BUF_LEN + 1];
		int n = -1;

		while ((n = bufin.read(cbuf, 0, BUF_LEN)) != -1) {
			strBuf.append(cbuf, 0, n);
		}

		bufin.close();
		stdin.close();

		return strBuf.toString();
	}

	private static String getExtendedPath(String[] args) {
		if (args == null || args.length < 1) {
			return "";
		} else {
			return args[0];
		}
	}

	private static Map getParams(String query, String enc) throws IOException {
		Map params = new HashMap();

		if (query == null || query.trim().equals("")) {
			return params;
		}

		String[] pairs = query.split("&");

		for (int i = 0; i < pairs.length; i++) {
			String pair = pairs[i];
			String[] name_value = pair.split("=");
			String name = name_value[0];
			String value = (name_value.length > 1) ? name_value[1] : "";

			if (enc != null) {
				value = URLDecoder.decode(value, enc);
			}

			if (params.containsKey(name)) {
				Object values = params.get(name);

				if (values instanceof List) {
					((List) values).add(value);
				} else {
					List list = new ArrayList();
					list.add(values);
					list.add(value);
					values = list;
				}

				params.put(name, values);
			} else {
				params.put(name, value);
			}
		}

		return params;
	}

}